@@ -9,7 +9,7 @@ use RDF::Trine::Serializer::NTriples;
 use RDF::Trine::Serializer::RDFXML;
 use Log::Log4perl qw(:easy);
 use Plack::Response;
-use RDF::LinkedData::Predicates;
+use RDF::Helper::Properties;
 
 with 'MooseX::Log::Log4perl::Easy';
 
@@ -26,43 +26,60 @@ RDF::LinkedData::ProviderRole - Role providing important functionality for Linke
 
 =head1 VERSION
 
-Version 0.05
+Version 0.09_1
 
 =cut
 
-our $VERSION = '0.05';
+our $VERSION = '0.09_1';
 
 
 =head1 SYNOPSIS
 
-From the L<Mojolicious::Lite> example:
+See L<RDF::LinkedData> for default usage.
 
-    my $ld = RDF::LinkedData->new($config->{store}, $config->{base});
 
-    my $uri = $self->param('uri');
-    my $type =  $self->param('type');
-    my $node = $ld->my_node($uri);
+=head1 DISCUSSION
 
-    if ($ld->count($node) > 0) {
-        my $content = $ld->content($node, $type);
-        $self->res->headers->header('Vary' => join(", ", qw(Accept)));
-        $self->res->headers->content_type($content->{content_type});
-        $self->render_text($content->{body});
-    } else {
-        $self->render_not_found;
-    }
+This module is now a L<Moose::Role>. The intention with this role is threefold:
+
+=over
+
+=item * This module may run standalone, in which case the default
+implementation in this role should be sufficient for a working Linked
+Data server. The empty L<RDF::LinkedData> class should provide such a
+default implementation.
+
+=item * This role may be implemented in classes that need to change
+some parts of its functionality, such as a L<mod_perl>-based server.
+
+=item * It may be a part of a larger server framework, for example a
+server that supports the SPARQL protocol and the SPARQL RESTful
+protocol.
+
+=back
+
+It is not completely clear at this point what the requirements are for
+these three scenarios, but it currently satisfies the first
+scenario. Thus, the role may need to be changed substantially and
+possibly split into different roles based on the usage that evolves
+over time.
+
+Consequently, one should not rely in the current API unless you are
+planning to keep track of the development of this module. It is still
+very much in flux, and may change without warning.
 
 
 =head1 METHODS
 
 =over
 
-=item C<< new ( config => $config, model => $model, base => $base, headers_in => $headers_in ) >>
+=item C<< new ( store => $store, model => $model, base => $base, headers_in => $headers_in ) >>
 
-Creates a new handler object based on named parameters, given a config
-string or model and a base URI. Optionally, you may pass a Apache
-request object, and you will need to pass a L<HTTP::Headers> object if
-you plan to call C<content>.
+Creates a new handler object based on named parameters, given a store
+config (which is a hashref that can be passed to
+L<RDF::Trine::Store>->new_with_config) or model and a base
+URI. Optionally, you may pass a Apache request object, and you will
+need to pass a L<HTTP::Headers> object if you plan to call C<content>.
 
 =cut
 
@@ -70,16 +87,16 @@ sub BUILD {
 	my $self = shift;
 
         unless($self->model) {
-            my $store	= RDF::Trine::Store->new_with_string( $self->config );
+            my $store	= RDF::Trine::Store->new_with_config( $self->store );
             $self->model(RDF::Trine::Model->new( $store ));
 	}
 
-        throw Error -text => "No valid RDF::Trine::Model, need either a config string or a model." unless ($self->model);
+        throw Error -text => "No valid RDF::Trine::Model, need either a store config string or a model." unless ($self->model);
 
 }
 
 
-has config => (is => 'rw', isa => 'Str' );
+has store => (is => 'rw', isa => 'HashRef' );
 
 
 =item C<< headers_in ( [ $headers ] ) >>
@@ -94,11 +111,25 @@ sub _build_headers_in {
     return HTTP::Headers->new() ;
 }
 
+=item C<< helper_properties (  ) >>
+
+Returns the L<RDF::Helper::Properties> object if it exists or sets
+it if a L<RDF::Helper::Properties> object is given as parameter.
+
+=cut
+
+has helper_properties => ( is => 'rw', isa => 'RDF::Helper::Properties', lazy => 1, builder => '_build_helper_properties');
+
+sub _build_helper_properties {
+    my $self = shift;
+    return RDF::Helper::Properties->new(model => $self->model);
+}
+
 
 
 =item C<< type >>
 
-Returns the chosen variant based on acceptable formats.
+Returns or sets the type of result to return, i.e. C<page>, in the case of a human-intended page or C<data> for machine consumption, or an empty string if it is an actual resource URI that should be redirected.
 
 =cut
 
@@ -109,7 +140,7 @@ has 'type' => (is => 'rw', isa => 'Str', default => '');
 
 =item C<< my_node >>
 
-A node for the requested relative URI. This node is typically used as
+A node for the requested B<relative> URI. This node is typically used as
 the subject to find which statements to return as data. Note that the
 base URI, set in the constructor or using the C<base> method, is
 prepended to the argument.
@@ -149,6 +180,13 @@ returned. The returned hashref has two keys: C<content_type> and
 C<body>. The former is self-explanatory, the latter contains the
 actual content.
 
+One may argue that a hashref with magic keys should be a class of its
+own, and for that reason, this method should be considered "at
+risk". Currently, it is only used in one place, and it may be turned
+into a private method, get passed the L<Plack::Response> object,
+removed altogether or turned into a role of its own, depending on the
+actual use cases that surfaces in the future.
+
 =cut
 
 
@@ -158,13 +196,14 @@ sub content {
     my %output;
     if ($type eq 'data') {
         $self->{_type} = 'data';
-        my ($type, $s) = RDF::Trine::Serializer->negotiate('request_headers' => $self->headers_in);
+        my ($type, $s) = RDF::Trine::Serializer->negotiate('request_headers' => $self->headers_in, 
+                                                           namespaces => $self->namespaces);
         my $iter = $model->bounded_description($node);
         $output{content_type} = $type;
         $output{body} = $s->serialize_iterator_to_string ( $iter );
     } else {
         $self->{_type} = 'page';
-        my $preds = RDF::LinkedData::Predicates->new($model);
+        my $preds = $self->helper_properties;
         my $title		= $preds->title( $node );
         my $desc		= $preds->description( $node );
         my $description	= sprintf( "<table>%s</table>\n", join("\n\t\t", map { sprintf( '<tr><td>%s</td><td>%s</td></tr>', @$_ ) } @$desc) );
@@ -215,7 +254,7 @@ has base => (is => 'rw', isa => 'Str', default => "http://localhost:3000" );
 
 =item C<< response ( $uri ) >>
 
-Will look up what to with the given URI and populate the response object.
+Will look up what to do with the given URI and populate the response object.
 
 =cut
 
@@ -229,8 +268,7 @@ sub response {
     $self->logger->info("Try rendering '$type' page for subject node: " . $node->as_string);
     if ($self->count($node) > 0) {
         if ($type) {
-            my $preds = RDF::LinkedData::Predicates->new($self->model);
-            
+            my $preds = $self->helper_properties;
             my $page = $preds->page($node);
             if (($type eq 'page') && ($page ne $node->uri_value . '/page')) {
                 # Then, we have a foaf:page set that we should redirect to
@@ -262,7 +300,7 @@ sub response {
             }
             my $newurl = $self->base . $uri . '/data';
             unless ($ct =~ /rdf|turtle/) {
-                my $preds = RDF::LinkedData::Predicates->new($self->model);
+                my $preds = $self->helper_properties;
                 $newurl = $preds->page($node);
             }
             $self->logger->debug('Will do a 303 redirect to ' . $newurl);
@@ -284,13 +322,25 @@ sub response {
 }
 
 
+=item namespaces ( { skos => 'http://www.w3.org/2004/02/skos/core#', dct => 'http://purl.org/dc/terms/' } )
+
+Gets or sets the namespaces that some serializers use for pretty-printing.
+
+=cut
+
+
+
+has 'namespaces' => (is => 'rw', isa => 'HashRef', default => sub { { rdf => 'http://www.w3.org/1999/02/22-rdf-syntax-ns#' } } );
+
 
 =back
 
 
 =head1 AUTHOR
 
-Most of the code was written by Gregory Todd Williams C<< <gwilliams@cpan.org> >> for L<RDF::LinkedData::Apache>, but refactored into this class for use by other modules by Kjetil Kjernsmo, C<< <kjetilk at cpan.org> >>
+This module was started by by Gregory Todd Williams C<<
+<gwilliams@cpan.org> >> for L<RDF::LinkedData::Apache>, but heavily
+refactored and rewritten by by Kjetil Kjernsmo, C<< <kjetilk@cpan.org> >>
 
 =head1 BUGS
 
@@ -299,36 +349,15 @@ the web interface at L<http://rt.cpan.org/NoAuth/ReportBug.html?Queue=RDF-Linked
 automatically be notified of progress on your bug as I make changes.
 
 
-
-
 =head1 SUPPORT
 
 You can find documentation for this module with the perldoc command.
 
-    perldoc RDF::LinkedData
+    perldoc RDF::LinkedData::ProviderRole
 
+The perlrdf mailing list is the right place to seek help and discuss this module:
 
-You can also look for information at:
-
-=over 4
-
-=item * RT: CPAN's request tracker
-
-L<http://rt.cpan.org/NoAuth/Bugs.html?Dist=RDF-LinkedData>
-
-=item * AnnoCPAN: Annotated CPAN documentation
-
-L<http://annocpan.org/dist/RDF-LinkedData>
-
-=item * CPAN Ratings
-
-L<http://cpanratings.perl.org/d/RDF-LinkedData>
-
-=item * Search CPAN
-
-L<http://search.cpan.org/dist/RDF-LinkedData>
-
-=back
+L<http://lists.perlrdf.org/listinfo/dev>
 
 
 =head1 ACKNOWLEDGEMENTS
@@ -344,4 +373,4 @@ under the same terms as Perl itself.
 
 =cut
 
-1; # End of RDF::LinkedData
+1;
